import os
from unittest.mock import MagicMock, patch


def test_run(cli):
    # Mocking the _register_subcommands and _dispatch_command methods
    cli._register_subcommands = MagicMock()
    cli._dispatch_command = MagicMock()

    # Mock argparse.ArgumentParser
    with patch("argparse.ArgumentParser") as MockArgumentParser:
        mock_parser = MagicMock()
        MockArgumentParser.return_value = mock_parser

        # Simulate a successful parse_args call with the expected arguments
        mock_parser.parse_args.return_value = type("Args", (), {"command": "create"})

        # Set the environment variable CLI_NAME (optional)
        with patch.dict(os.environ, {"CLI_NAME": "TestCLI"}):
            cli.run()

        # Check if the _register_subcommands and _dispatch_command were called
        cli._register_subcommands.assert_called_once()
        cli._dispatch_command.assert_called_once_with(
            mock_parser.parse_args.return_value
        )

        # Verify the correct arguments passed to ArgumentParser
        mock_parser.add_subparsers.assert_called_once_with(
            dest="command", required=True
        )
        mock_parser.parse_args.assert_called_once()



from unittest.mock import MagicMock, patch

from rest.client import JiraClient


def test_request_success():
    client = JiraClient()
    # Mock a successful response
    mock_response = MagicMock()
    mock_response.status_code = 200
    mock_response.text = '{"key": "value"}'
    # Mock json method to return the actual dictionary
    mock_response.json.return_value = {"key": "value"}

    with patch("requests.request", return_value=mock_response) as mock_request:
        # Call the method that should trigger the mock
        result = client._request("GET", "/rest/api/2/issue/ISSUE-123")

        # Assert that the correct result is returned
        assert result == {"key": "value"}

        # Verify that requests.request was called with the correct arguments
        mock_request.assert_called_once()



def test_set_priority(client):
    # Call the method to set priority
    client.set_priority("AAP-test_set_priority", "High")

    # Update the test to expect the 'allow_204' argument
    client._request.assert_called_once_with(
        "PUT",
        "/rest/api/2/issue/AAP-test_set_priority",
        json={"fields": {"priority": {"name": "High"}}},
    )



import os
from enum import Enum


class IssueType(Enum):
    BUG = "bug"
    EPIC = "epic"
    SPIKE = "spike"
    STORY = "story"
    TASK = "task"
    COMMENT = "comment"
    DEFAULT = "default"
    QC = "qc"


JIRA_FORMATTING_ISSUES = (
    "Be concerned with jira formatting issues.  Some items to consider are: "
    "Convert '###' to 'h4.' and convert '##' to 'h3.' and convert '#' into 'h2.'\n\n'"
    "If there is a number before the hashes remove it.\n\n"
    "If you see a code block, something like ```yaml  some code here ```  convert it into "
    "{code:java}this format{code}\n\n"
    "Other formatting issues include the use of '**', e.g: **some text**.  This is"
    " meant to be text surrounded by a single '*' either side to indicate bold.\n\n"
    "If you spot other jira formatting issues, be sure to clean them up.\n\n"
)

BASE_PROMPT = (
    "You are a professional Principal Software Engineer. You write acute, "
    "well-defined Jira {type}s with a strong focus on clarity, structure, and "
    "detail.\n"
    "If standard Jira sections are missing, add them—such as Description, "
    "Definition of Done, and Acceptance Criteria. If these sections already "
    "exist, preserve and clean up their format.\n"
    "Focus on fixing spelling errors, correcting grammatical issues, and "
    "improving sentence readability for greater clarity.\n\n"
    "Follow this structure:\n\n"
)

TEMPLATE_DIR = os.path.join(os.path.dirname(__file__), "../templates")


class PromptLibrary:
    @staticmethod
    def get_file_contents(full_name):
        template = ""
        template_path = os.path.join(TEMPLATE_DIR, f"{full_name}.tmpl")

        if not os.path.exists(template_path):
            raise FileNotFoundError(f"Template not found: {template_path}")

        with open(template_path, "r", encoding="utf-8") as f:
            template = f.read().strip()

        return template

    @staticmethod
    def get_prompt(issue_type: IssueType) -> str:
        # Check if the issue_type is "comment" first
        prompt = ""
        full_name = issue_type.value.lower()

        if issue_type == IssueType.DEFAULT:
            prompt = (
                JIRA_FORMATTING_ISSUES
                + BASE_PROMPT.format(type=issue_type.value)
                + PromptLibrary.get_file_contents(full_name)
            )

        elif issue_type == IssueType.COMMENT:
            prompt = (
                "As a professional Principal Software Engineer, you write great "
                "comments on jira issues. As you are just writing comments, whilst "
                "being clear you don't need to heavily structure the messages. "
                "Improve clarity, fix grammar and spelling, and maintain structure."
            ) + JIRA_FORMATTING_ISSUES

        elif issue_type == IssueType.QC:
            prompt = PromptLibrary.get_file_contents(full_name)

        # Check if issue_type is one of the IssueType enum members
        elif issue_type in [issue_type for issue_type in IssueType]:
            prompt = (
                "As a professional Principal Software Engineer, you write acute, "
                "well-defined Jira issues with a strong focus on clear descriptions, "
                "definitions of done, acceptance criteria, and supporting details. "
                "If standard Jira sections are missing, add them. Improve clarity, "
                "fix grammar and spelling, and maintain structure."
            ) + JIRA_FORMATTING_ISSUES

        return prompt



from unittest.mock import MagicMock, patch

import pytest
from exceptions.exceptions import UpdateDescriptionError


@patch("commands.cli_edit_issue.subprocess.call", return_value=0)
@patch("commands.cli_edit_issue.tempfile.NamedTemporaryFile")
def test_edit_issue_update_exception(mock_tmpfile, mock_subprocess, capsys, cli):
    # Mock Jira internals
    cli.jira.get_description = MagicMock(return_value="original")
    cli.jira.get_issue_type = MagicMock(return_value="story")
    cli.jira.update_description = MagicMock(side_effect=UpdateDescriptionError("fail"))

    # Mock cleanup logic
    cli._try_cleanup = MagicMock(return_value="cleaned")
    cli.ai_provider.improve_text = MagicMock(return_value="cleaned")  # ✅ Important

    # Mock temp file
    fake_file = MagicMock()
    fake_file.__enter__.return_value = fake_file
    fake_file.read.return_value = "edited"
    fake_file.write = MagicMock()
    fake_file.flush = MagicMock()
    fake_file.seek = MagicMock()
    fake_file.name = "/tmp/fake_edit"
    mock_tmpfile.return_value = fake_file

    # Simulated CLI args
    class Args:
        issue_key = "AAP-test_edit_issue_update_exception"
        no_ai = False
        lint = False  # ✅ Add this to fix the error

    with pytest.raises(UpdateDescriptionError):
        cli.edit_issue(Args())

    out = capsys.readouterr().out
    assert "❌ Update failed" in out
